@@ -1,8 +1,7 @@
-#
-# $Id: Headers.pm,v 1.37 1999/10/28 12:09:12 gisle Exp $
-
 package HTTP::Headers;
 
+# $Id: Headers.pm,v 1.44 2002/06/29 00:41:29 gisle Exp $
+
 =head1 NAME
 
 HTTP::Headers - Class encapsulating HTTP Message headers
@@ -10,13 +9,17 @@ HTTP::Headers - Class encapsulating HTTP Message headers
 =head1 SYNOPSIS
 
  require HTTP::Headers;
- $h = new HTTP::Headers;
+ $h = HTTP::Headers->new;
+
+ $h->header('Content-Type' => 'text/plain');  # set
+ $ct = $h->header('Content-Type');            # get
+ $h->remove_header('Content-Type');           # delete
 
 =head1 DESCRIPTION
 
 The C<HTTP::Headers> class encapsulates HTTP-style message headers.
-The headers consist of attribute-value pairs, which may be repeated,
-and which are printed in a particular order.
+The headers consist of attribute-value pairs also called fields, which
+may be repeated, and which are printed in a particular order.
 
 Instances of this class are usually created as member variables of the
 C<HTTP::Request> and C<HTTP::Response> classes, internal to the
@@ -29,63 +32,66 @@ The following methods are available:
 =cut
 
 use strict;
-use vars qw($VERSION $TRANSLATE_UNDERSCORE);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
-
 use Carp ();
 
-# Could not use the AutoLoader becase several of the method names are
-# not unique in the first 8 characters.
-#use SelfLoader;
+use vars qw($VERSION $TRANSLATE_UNDERSCORE);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.44 $ =~ /(\d+)\.(\d+)/);
 
+# The $TRANSLATE_UNDERSCORE variable controls whether '_' can be used
+# as a replacement for '-' in header field names.
+$TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
 
 # "Good Practice" order of HTTP message headers:
 #    - General-Headers
 #    - Request-Headers
 #    - Response-Headers
 #    - Entity-Headers
-# (From draft-ietf-http-v11-spec-rev-01, Nov 21, 1997)
 
 my @header_order = qw(
-   Cache-Control Connection Date Pragma Transfer-Encoding Upgrade Trailer Via
+   Cache-Control Connection Date Pragma Trailer Transfer-Encoding Upgrade
+   Via Warning
 
    Accept Accept-Charset Accept-Encoding Accept-Language
    Authorization Expect From Host
-   If-Modified-Since If-Match If-None-Match If-Range If-Unmodified-Since
+   If-Match If-Modified-Since If-None-Match If-Range If-Unmodified-Since
    Max-Forwards Proxy-Authorization Range Referer TE User-Agent
 
-   Accept-Ranges Age Location Proxy-Authenticate Retry-After Server Vary
-   Warning WWW-Authenticate
+   Accept-Ranges Age ETag Location Proxy-Authenticate Retry-After Server
+   Vary WWW-Authenticate
 
-   Allow Content-Base Content-Encoding Content-Language Content-Length
-   Content-Location Content-MD5 Content-Range Content-Type
-   ETag Expires Last-Modified
+   Allow Content-Encoding Content-Language Content-Length Content-Location
+   Content-MD5 Content-Range Content-Type Expires Last-Modified
 );
 
 # Make alternative representations of @header_order.  This is used
 # for sorting and case matching.
-my $i = 0;
 my %header_order;
 my %standard_case;
-for (@header_order) {
-    my $lc = lc $_;
-    $header_order{$lc} = $i++;
-    $standard_case{$lc} = $_;
+
+{
+    my $i = 0;
+    for (@header_order) {
+	my $lc = lc $_;
+	$header_order{$lc} = ++$i;
+	$standard_case{$lc} = $_;
+    }
 }
 
-$TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
 
 
 
-=item $h = new HTTP::Headers
+=item $h = HTTP::Headers->new
 
 Constructs a new C<HTTP::Headers> object.  You might pass some initial
 attribute-value pairs as parameters to the constructor.  I<E.g.>:
 
- $h = new HTTP::Headers
-     Date         => 'Thu, 03 Feb 1994 00:00:00 GMT',
-     Content_Type => 'text/html; version=3.2',
-     Content_Base => 'http://www.sn.no/';
+ $h = HTTP::Headers->new(
+       Date         => 'Thu, 03 Feb 1994 00:00:00 GMT',
+       Content_Type => 'text/html; version=3.2',
+       Content_Base => 'http://www.perl.org/');
+
+The constructor arguments are passed to the C<header> method which is
+described below.
 
 =cut
 
@@ -100,36 +106,47 @@ sub new
 
 =item $h->header($field [=> $value],...)
 
-Get or set the value of a header.  The header field name is not case
-sensitive.  To make the life easier for perl users who wants to avoid
-quoting before the => operator, you can use '_' as a synonym for '-'
-in header names (this behaviour can be suppressed by setting
-$HTTP::Headers::TRANSLATE_UNDERSCORE to a FALSE value).
+Get or set the value of one or more header fields.  The header field name
+($field) is not case sensitive.  To make the life easier for perl
+users who wants to avoid quoting before the => operator, you can use
+'_' as a replacement for '-' in header names (this behaviour can be
+suppressed by setting the $HTTP::Headers::TRANSLATE_UNDERSCORE
+variable to a FALSE value).
 
-The header() method accepts multiple ($field => $value) pairs, so you
-can update several fields with a single invocation.
+The header() method accepts multiple ($field => $value) pairs, which
+means that you can update several fields with a single invocation.
 
-The optional $value argument may be a scalar or a reference to a list
-of scalars. If the $value argument is undefined or not given, then the
-header is not modified.
+The $value argument may be a plain string or a reference to an array
+of strings for a multi-valued field. If the $value is undefined or not
+given, then that header field will remain unchanged.
 
-The old value of the last of the $field values is returned.
-Multi-valued fields will be concatenated with "," as separator in
-scalar context.
+The old value (or values) of the last of the header fields is returned.
+If no such field exists C<undef> will be returned.
+
+A multi-valued field will be retuned as separate values in list
+context and will be concatenated with ", " as separator in scalar
+context.  The HTTP spec (RFC 2616) promise that joining multiple
+values in this way will not change the semantic of a header field, but
+in practice there are cases like old-style Netscape cookies (see
+L<HTTP::Cookies>) where "," is used as part of the syntax of a single
+field value.
+
+Examples:
 
  $header->header(MIME_Version => '1.0',
 		 User_Agent   => 'My-Web-Client/0.01');
  $header->header(Accept => "text/html, text/plain, image/*");
  $header->header(Accept => [qw(text/html text/plain image/*)]);
- @accepts = $header->header('Accept');
+ @accepts = $header->header('Accept');  # get multiple values
+ $accepts = $header->header('Accept');  # get values as a single string
 
 =cut
 
 sub header
 {
     my $self = shift;
-    my($field, $val, @old);
-    while (($field, $val) = splice(@_, 0, 2)) {
+    my(@old);
+    while (my($field, $val) = splice(@_, 0, 2)) {
 	@old = $self->_header($field, $val);
     }
     return @old if wantarray;
@@ -137,9 +154,81 @@ sub header
     join(", ", @old);
 }
 
+
+=item $h->push_header($field, $value)
+
+Add a new field value for the specified header field.  Previous values
+for the same field are retained.
+
+As for the header() method, the field name ($field) is not case
+sensitive and '_' can be used as a replacement for '-'.
+
+The $value argument may be a scalar or a reference to a list of
+scalars.
+
+ $header->push_header(Accept => 'image/jpeg');
+ $header->push_header(Accept => [map "image/$_", qw(gif png tiff)]);
+
+=cut
+
+sub push_header
+{
+    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
+    shift->_header(@_, 'PUSH');
+}
+
+=item $h->init_header($field, $value)
+
+Set the specified header to the given value, but only if no previous
+value for that field is set.
+
+The header field name ($field) is not case sensitive and '_'
+can be used as a replacement for '-'.
+
+The $value argument may be a scalar or a reference to a list of
+scalars.
+
+=cut
+
+sub init_header
+{
+    Carp::croak('Usage: $h->init_header($field, $val)') if @_ != 3;
+    shift->_header(@_, 'INIT');
+}
+
+
+=item $h->remove_header($field,...)
+
+This function removes the headers fields with the specified names.
+
+The header field names ($field) are not case sensitive and '_'
+can be used as a replacement for '-'.
+
+The return value is the values of the fields removed.  In scalar
+context the number of fields removed is returned.
+
+Note that if you pass in multiple field names then it is generally not
+possible to tell which of the returned values belonged to which field.
+
+=cut
+
+sub remove_header
+{
+    my($self, @fields) = @_;
+    my $field;
+    my @values;
+    foreach $field (@fields) {
+	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
+	my $v = delete $self->{lc $field};
+	push(@values, ref($v) eq 'ARRAY' ? @$v : $v) if defined $v;
+    }
+    return @values;
+}
+
+
 sub _header
 {
-    my($self, $field, $val, $push) = @_;
+    my($self, $field, $val, $op) = @_;
     $field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
 
     # $push is only used interally sub push_header
@@ -147,22 +236,23 @@ sub _header
 
     my $lc_field = lc $field;
     unless(defined $standard_case{$lc_field}) {
-	# generate a %stadard_case entry for this field
+	# generate a %standard_case entry for this field
 	$field =~ s/\b(\w)/\u$1/g;
 	$standard_case{$lc_field} = $field;
     }
 
     my $h = $self->{$lc_field};
-    my @old = ref($h) ? @$h : (defined($h) ? ($h) : ());
+    my @old = ref($h) eq 'ARRAY' ? @$h : (defined($h) ? ($h) : ());
 
-    if (defined $val) {
-	my @new = $push ? @old : ();
-	if (!ref($val)) {
+    $op ||= "";
+    $val = undef if $op eq 'INIT' && @old;
+    if (defined($val)) {
+	my @new = ($op eq 'PUSH') ? @old : ();
+	if (ref($val) ne 'ARRAY') {
 	    push(@new, $val);
-	} elsif (ref($val) eq 'ARRAY') {
+	}
+	else {
 	    push(@new, @$val);
-	} else {
-	    Carp::croak("Unexpected field value $val");
 	}
 	$self->{$lc_field} = @new > 1 ? \@new : $new[0];
     }
@@ -174,24 +264,21 @@ sub _header
 # recommended "Good Practice" order.
 sub _header_cmp
 {
-    # Unknown headers are assign a large value so that they are
-    # sorted last.  This also helps avoiding a warning from -w
-    # about comparing undefined values.
-    $header_order{$a} = 999 unless defined $header_order{$a};
-    $header_order{$b} = 999 unless defined $header_order{$b};
-
-    $header_order{$a} <=> $header_order{$b} || $a cmp $b;
+    ($header_order{$a} || 999) <=> ($header_order{$b} || 999) || $a cmp $b;
 }
 
 
 =item $h->scan(\&doit)
 
-Apply a subroutine to each header in turn.  The callback routine is
-called with two parameters; the name of the field and a single value.
-If the header has more than one value, then the routine is called once
-for each value.  The field name passed to the callback routine has
-case as suggested by HTTP Spec, and the headers will be visited in the
-recommended "Good Practice" order.
+Apply a subroutine to each header field in turn.  The callback routine
+is called with two parameters; the name of the field and a single
+value (a string).  If a header field is multi-valued, then the
+routine is called once for each value.  The field name passed to the
+callback routine has case as suggested by HTTP spec, and the headers
+will be visited in the recommended "Good Practice" order.
+
+Any return values of the callback routine are ignored.  The loop can
+be broken by raising an exception (C<die>).
 
 =cut
 
@@ -202,7 +289,7 @@ sub scan
     foreach $key (sort _header_cmp keys %$self) {
         next if $key =~ /^_/;
 	my $vals = $self->{$key};
-	if (ref($vals)) {
+	if (ref($vals) eq 'ARRAY') {
 	    my $val;
 	    for $val (@$vals) {
 		&$sub($standard_case{$key} || $key, $val);
@@ -217,14 +304,14 @@ sub scan
 =item $h->as_string([$endl])
 
 Return the header fields as a formatted MIME header.  Since it
-internally uses the C<scan()> method to build the string, the result
-will use case as suggested by HTTP Spec, and it will follow
+internally uses the C<scan> method to build the string, the result
+will use case as suggested by HTTP spec, and it will follow
 recommended "Good Practice" of ordering the header fieds.  Long header
-values are not folded. 
+values are not folded.
 
-The optional parameter specifies the line ending sequence to use.  The
-default is C<"\n">.  Embedded "\n" characters in the header will be
-substitued with this line ending sequence.
+The optional $endl parameter specifies the line ending sequence to
+use.  The default is "\n".  Embedded "\n" characters in header field
+values will be substitued with this line ending sequence.
 
 =cut
 
@@ -250,37 +337,30 @@ sub as_string
 }
 
 
-# The remaining functions should autoloaded only when needed
-
-# A bug in 5.002gamma makes it risky to have POD text inside the
-# autoloaded section of the code, so we keep the documentation before
-# the __DATA__ token.
-
-=item $h->push_header($field, $val)
-
-Add a new field value of the specified header.  The header field name
-is not case sensitive.  The field need not already have a
-value. Previous values for the same field are retained.  The argument
-may be a scalar or a reference to a list of scalars.
-
- $header->push_header(Accept => 'image/jpeg');
-
-=item $h->remove_header($field,...)
-
-This function removes the headers with the specified names.
-
 =item $h->clone
 
-Returns a copy of this HTTP::Headers object.
+Returns a copy of this C<HTTP::Headers> object.
 
 =back
 
+=cut
+
+sub clone
+{
+    my $self = shift;
+    my $clone = new HTTP::Headers;
+    $self->scan(sub { $clone->push_header(@_);} );
+    $clone;
+}
+
+
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
 following convenience methods.  These methods can both be used to read
 and to set the value of a header.  The header value is set if you pass
 an argument to the method.  The old header value is always returned.
+If the given header did not exists then C<undef> is returned.
 
 Methods that deal with dates/times always convert their value to system
 time (seconds since Jan 1, 1970) and they also expect this kind of
@@ -304,9 +384,9 @@ considered stale.
 
 =item $h->if_unmodified_since
 
-This header is used to make a request conditional.  If the requested
-resource has (not) been modified since the time specified in this field,
-then the server will return a C<"304 Not Modified"> response instead of
+These header fields are used to make a request conditional.  If the requested
+resource has (or has not) been modified since the time specified in this field,
+then the server will return a C<304 Not Modified> response instead of
 the document itself.
 
 =item $h->last_modified
@@ -315,8 +395,10 @@ This header indicates the date and time at which the resource was last
 modified. I<E.g.>:
 
   # check if document is more than 1 hour old
-  if ($h->last_modified < time - 60*60) {
-	...
+  if (my $last_mod = $h->last_modified) {
+      if ($last_mod < time - 60*60) {
+	  ...
+      }
   }
 
 =item $h->content_type
@@ -350,7 +432,8 @@ A decimal number indicating the size in bytes of the message content.
 
 The natural language(s) of the intended audience for the message
 content.  The value is one or more language tags as defined by RFC
-1766.  Eg. "no" for Norwegian and "en-US" for US-English.
+1766.  Eg. "no" for some kind of Norwegian and "en-US" for English the
+way it is written in the US.
 
 =item $h->title
 
@@ -377,22 +460,41 @@ This header should contain an Internet e-mail address for the human
 user who controls the requesting user agent.  The address should be
 machine-usable, as defined by RFC822.  E.g.:
 
-  $h->from('Gisle Aas <aas@sn.no>');
+  $h->from('King Kong <king@kong.com>');
+
+I<This header is no longer part of the HTTP standard.>
 
 =item $h->referer
 
 Used to specify the address (URI) of the document from which the
 requested resouce address was obtained.
 
+The "Free On-line Dictionary of Computing" as this to say about the
+word I<referer>:
+
+     <World-Wide Web> A misspelling of "referrer" which
+     somehow made it into the {HTTP} standard.  A given {web
+     page}'s referer (sic) is the {URL} of whatever web page
+     contains the link that the user followed to the current
+     page.  Most browsers pass this information as part of a
+     request.
+
+     (1998-10-19)
+
+By popular demand C<referrer> exists as an alias for this method so you
+can avoid this misspelling in your programs and still send the right
+thing on the wire.
+
+
 =item $h->www_authenticate
 
-This header must be included as part of a "401 Unauthorized" response.
+This header must be included as part of a C<401 Unauthorized> response.
 The field value consist of a challenge that indicates the
 authentication scheme and parameters applicable to the requested URI.
 
 =item $h->proxy_authenticate
 
-This header must be included in a "407 Proxy Authentication Required"
+This header must be included in a C<407 Proxy Authentication Required>
 response.
 
 =item $h->authorization
@@ -422,46 +524,8 @@ header instead.
 
 =back
 
-=head1 COPYRIGHT
-
-Copyright 1995-1998 Gisle Aas.
-
-This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
 =cut
 
-1;
-
-#__DATA__
-
-sub clone
-{
-    my $self = shift;
-    my $clone = new HTTP::Headers;
-    $self->scan(sub { $clone->push_header(@_);} );
-    $clone;
-}
-
-sub push_header
-{
-    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
-    shift->_header(@_, 'PUSH');
-}
-
-
-sub remove_header
-{
-    my($self, @fields) = @_;
-    my $field;
-    foreach $field (@fields) {
-	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
-	delete $self->{lc $field};
-    }
-}
-
-# Convenience access functions
-
 sub _date_header
 {
     require HTTP::Date;
@@ -507,8 +571,8 @@ sub server            { (shift->_header('Server',           @_))[0] }
 
 sub from              { (shift->_header('From',             @_))[0] }
 sub referer           { (shift->_header('Referer',          @_))[0] }
-sub warning           { (shift->_header('Warning',          @_))[0] }
 *referrer = \&referer;  # on tchrist's request
+sub warning           { (shift->_header('Warning',          @_))[0] }
 
 sub www_authenticate  { (shift->_header('WWW-Authenticate', @_))[0] }
 sub authorization     { (shift->_header('Authorization',    @_))[0] }
@@ -538,4 +602,13 @@ sub _basic_auth {
     return;
 }
 
+=head1 COPYRIGHT
+
+Copyright 1995-2002 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
+
 1;
