@@ -1,13 +1,13 @@
 package Net::HTTP::Methods;
 
-# $Id: Methods.pm,v 1.4 2001/11/20 20:46:11 gisle Exp $
+# $Id: Methods.pm,v 1.10 2002/12/20 08:32:59 gisle Exp $
 
 require 5.005;  # 4-arg substr
 
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "0.01";
+$VERSION = "0.02";
 
 my $CRLF = "\015\012";   # "\r\n" is not portable
 
@@ -170,13 +170,13 @@ sub write_request {
 sub format_chunk {
     my $self = shift;
     return $_[0] unless defined($_[0]) && length($_[0]);
-    return hex(length($_[0])) . $CRLF . $_[0] . $CRLF;
+    return sprintf("%x", length($_[0])) . $CRLF . $_[0] . $CRLF;
 }
 
 sub write_chunk {
     my $self = shift;
     return 1 unless defined($_[0]) && length($_[0]);
-    $self->print(hex(length($_[0])) . $CRLF . $_[0] . $CRLF);
+    $self->print(sprintf("%x", length($_[0])) . $CRLF . $_[0] . $CRLF);
 }
 
 sub format_chunk_eof {
@@ -233,8 +233,8 @@ sub my_readline {
 	    if $max_line_length && $pos > $max_line_length;
 
 	my $line = substr($_, 0, $pos+1, "");
-	$line =~ s/\015?\012\z//;
-	return $line;
+	$line =~ s/(\015?\012)\z// || die "Assert";
+	return wantarray ? ($line, $1) : $line;
     }
 }
 
@@ -260,8 +260,10 @@ sub _rbuf_length {
 }
 
 
-sub read_header_lines {
+sub _read_header_lines {
     my $self = shift;
+    my $junk_out = shift;
+
     my @headers;
     my $line_count = 0;
     my $max_header_lines = ${*$self}{'http_max_header_lines'};
@@ -272,8 +274,11 @@ sub read_header_lines {
 	elsif (@headers && $line =~ s/^\s+//) {
 	    $headers[-1] .= " " . $line;
 	}
+	elsif ($junk_out) {
+	    push(@$junk_out, $line);
+	}
 	else {
-	    die "Bad header: $line\n";
+	    die "Bad header: '$line'\n";
 	}
 	if ($max_header_lines) {
 	    $line_count++;
@@ -287,15 +292,34 @@ sub read_header_lines {
 
 
 sub read_response_headers {
-    my $self = shift;
-    my $status = my_readline($self);
-    die "EOF instead of reponse status line" unless defined $status;
+    my($self, %opt) = @_;
+    my $laxed = $opt{laxed};
+
+    my($status, $eol) = my_readline($self);
+    die "EOF instead of response status line" unless defined $status;
+
     my($peer_ver, $code, $message) = split(/\s+/, $status, 3);
-    die "Bad response status line: '$status'"
-	if !$peer_ver || $peer_ver !~ s,^HTTP/,,;
+    if (!$peer_ver || $peer_ver !~ s,^HTTP/,,) {
+	die "Bad response status line: '$status'" unless $laxed;
+	# assume HTTP/0.9
+	${*$self}{'http_peer_http_version'} = "0.9";
+	${*$self}{'http_status'} = "200";
+	substr(${*$self}{'http_buf'}, 0, 0) = $status . $eol;
+	return (200, "Assumed OK");
+    };
+
     ${*$self}{'http_peer_http_version'} = $peer_ver;
+
+    unless ($code =~ /^[1-9]\d\d$/) {
+	die "Bad response code: '$status'";
+    }
     ${*$self}{'http_status'} = $code;
-    my @headers = $self->read_header_lines;
+
+    my $junk_out;
+    if ($laxed) {
+	$junk_out = $opt{junk_out} || [];
+    }
+    my @headers = $self->_read_header_lines($junk_out);
 
     # pick out headers that read_entity_body might need
     my @te;
@@ -395,14 +419,17 @@ sub read_entity_body {
 	if ($chunked <= 0) {
 	    my $line = my_readline($self);
 	    if ($chunked == 0) {
-		die "Not empty: '$line'" unless $line eq "";
+		die "Missing newline after chunk data: '$line'" unless $line eq "";
 		$line = my_readline($self);
 	    }
-	    $line =~ s/;.*//;  # ignore potential chunk parameters
-	    $line =~ s/\s+$//; # avoid warnings from hex()
-	    $chunked = hex($line);
+	    my $chunk_len = $line;
+	    $chunk_len =~ s/;.*//;  # ignore potential chunk parameters
+	    unless ($chunk_len =~ /^([\da-fA-F]+)\s*$/) {
+		die "Bad chunk-size in HTTP response: $line";
+	    }
+	    $chunked = hex($1);
 	    if ($chunked == 0) {
-		${*$self}{'http_trailers'} = [$self->read_header_lines];
+		${*$self}{'http_trailers'} = [$self->_read_header_lines];
 		$$buf_ref = "";
 
 		my $n = 0;
@@ -480,7 +507,8 @@ sub zlib_ok {
 	Compress::Zlib->VERSION(1.10);
 	$zlib_ok++;
     };
-    #warn $@ if $@ && $^W;
+
+    return $zlib_ok;
 }
 
 } # BEGIN
